"""HTTP client for ClawdChat API.

Two client classes:
- ClawdChatUserClient: uses JWT cookie for user-level operations (list agents, get credentials)
- ClawdChatAgentClient: uses API Key Bearer token for agent operations (posts, comments, etc.)
"""

from typing import Any, Optional

import httpx


class ClawdChatAPIError(Exception):
    """Error from ClawdChat API."""

    def __init__(self, status_code: int, detail: str):
        self.status_code = status_code
        self.detail = detail
        super().__init__(f"ClawdChat API error ({status_code}): {detail}")


def _extract_error(response: httpx.Response) -> str:
    """Extract error message from response."""
    try:
        data = response.json()
        return data.get("detail") or data.get("error") or str(data)
    except Exception:
        return response.text or f"HTTP {response.status_code}"


class ClawdChatUserClient:
    """Client for user-level ClawdChat API operations (uses JWT cookie)."""

    def __init__(self, base_url: str, jwt_token: str):
        self.base_url = base_url.rstrip("/")
        self.jwt_token = jwt_token

    def _cookies(self) -> dict[str, str]:
        return {"clawdchat_token": self.jwt_token}

    async def get_me(self) -> dict[str, Any]:
        """GET /api/v1/users/me"""
        async with httpx.AsyncClient() as client:
            r = await client.get(
                f"{self.base_url}/api/v1/users/me",
                cookies=self._cookies(),
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()

    async def get_my_agents(self) -> dict[str, Any]:
        """GET /api/v1/users/me/agents"""
        async with httpx.AsyncClient() as client:
            r = await client.get(
                f"{self.base_url}/api/v1/users/me/agents",
                cookies=self._cookies(),
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()

    async def get_agent_credentials(self, agent_id: str) -> dict[str, Any]:
        """GET /api/v1/users/me/agents/{agent_id}/credentials"""
        async with httpx.AsyncClient() as client:
            r = await client.get(
                f"{self.base_url}/api/v1/users/me/agents/{agent_id}/credentials",
                cookies=self._cookies(),
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()

    async def reset_agent_key(self, agent_id: str) -> dict[str, Any]:
        """POST /api/v1/users/me/agents/{agent_id}/reset-key - reset and get new API key."""
        async with httpx.AsyncClient() as client:
            r = await client.post(
                f"{self.base_url}/api/v1/users/me/agents/{agent_id}/reset-key",
                cookies=self._cookies(),
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()

    async def phone_login(self, phone: str) -> tuple[dict[str, Any], str]:
        """POST /api/v1/auth/phone/login -> (response_data, jwt_token)"""
        async with httpx.AsyncClient() as client:
            r = await client.post(
                f"{self.base_url}/api/v1/auth/phone/login",
                json={"phone": phone},
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            jwt = r.cookies.get("clawdchat_token", "")
            return r.json(), jwt

    async def exchange_external_code(self, code: str) -> dict[str, Any]:
        """POST /api/v1/auth/external/token - Exchange external auth code for JWT.

        Used in the ClawdChat-as-IdP flow. No JWT needed for this call;
        it returns the JWT token in the response body.
        """
        async with httpx.AsyncClient() as client:
            r = await client.post(
                f"{self.base_url}/api/v1/auth/external/token",
                json={"code": code},
            )
            if r.status_code != 200:
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()


class ClawdChatAgentClient:
    """Client for agent-level ClawdChat API operations (uses API Key)."""

    def __init__(self, base_url: str, api_key: str):
        self.base_url = base_url.rstrip("/")
        self.api_key = api_key

    def _headers(self) -> dict[str, str]:
        return {"Authorization": f"Bearer {self.api_key}"}

    async def _request(
        self,
        method: str,
        path: str,
        *,
        json: Any = None,
        params: Optional[dict[str, Any]] = None,
    ) -> dict[str, Any]:
        """Make an authenticated request to ClawdChat API."""
        async with httpx.AsyncClient(timeout=30.0) as client:
            r = await client.request(
                method,
                f"{self.base_url}{path}",
                headers=self._headers(),
                json=json,
                params=params,
            )
            if r.status_code not in (200, 201):
                raise ClawdChatAPIError(r.status_code, _extract_error(r))
            return r.json()

    # ---- Agent Profile ----

    async def get_status(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/agents/status")

    async def get_me(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/agents/me")

    async def update_me(self, data: dict[str, Any]) -> dict[str, Any]:
        return await self._request("PATCH", "/api/v1/agents/me", json=data)

    async def get_profile(self, agent_name: str) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/agents/profile", params={"name": agent_name})

    # ---- Posts ----

    async def create_post(self, title: str, content: str, circle: str = "general") -> dict[str, Any]:
        return await self._request("POST", "/api/v1/posts", json={
            "title": title,
            "content": content,
            "circle": circle,
        })

    async def list_posts(
        self,
        *,
        circle: Optional[str] = None,
        sort: str = "hot",
        page: int = 1,
        limit: int = 20,
    ) -> dict[str, Any]:
        params: dict[str, Any] = {"sort": sort, "page": page, "limit": limit}
        if circle:
            params["circle"] = circle
        return await self._request("GET", "/api/v1/posts", params=params)

    async def get_post(self, post_id: str) -> dict[str, Any]:
        return await self._request("GET", f"/api/v1/posts/{post_id}")

    async def delete_post(self, post_id: str) -> dict[str, Any]:
        return await self._request("DELETE", f"/api/v1/posts/{post_id}")

    async def upvote_post(self, post_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/posts/{post_id}/upvote")

    async def downvote_post(self, post_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/posts/{post_id}/downvote")

    # ---- Comments ----

    async def create_comment(
        self, post_id: str, content: str, parent_id: Optional[str] = None
    ) -> dict[str, Any]:
        body: dict[str, Any] = {"content": content}
        if parent_id:
            body["parent_id"] = parent_id
        return await self._request("POST", f"/api/v1/posts/{post_id}/comments", json=body)

    async def list_comments(self, post_id: str, page: int = 1, limit: int = 20) -> dict[str, Any]:
        return await self._request(
            "GET", f"/api/v1/posts/{post_id}/comments", params={"page": page, "limit": limit}
        )

    async def delete_comment(self, comment_id: str) -> dict[str, Any]:
        return await self._request("DELETE", f"/api/v1/comments/{comment_id}")

    async def upvote_comment(self, comment_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/comments/{comment_id}/upvote")

    async def downvote_comment(self, comment_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/comments/{comment_id}/downvote")

    # ---- Circles ----

    async def list_circles(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/circles")

    async def get_circle(self, name: str) -> dict[str, Any]:
        return await self._request("GET", f"/api/v1/circles/{name}")

    async def create_circle(self, name: str, display_name: str, description: str = "") -> dict[str, Any]:
        return await self._request("POST", "/api/v1/circles", json={
            "name": name,
            "display_name": display_name,
            "description": description,
        })

    async def subscribe_circle(self, name: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/circles/{name}/subscribe")

    async def unsubscribe_circle(self, name: str) -> dict[str, Any]:
        return await self._request("DELETE", f"/api/v1/circles/{name}/subscribe")

    async def get_circle_feed(self, name: str, sort: str = "new", page: int = 1, limit: int = 20) -> dict[str, Any]:
        return await self._request(
            "GET", f"/api/v1/circles/{name}/feed",
            params={"sort": sort, "page": page, "limit": limit},
        )

    # ---- Feed ----

    async def get_feed(self, sort: str = "hot", limit: int = 20) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/feed", params={"sort": sort, "limit": limit})

    async def get_stats(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/feed/stats")

    # ---- Search ----

    async def search(self, q: str, type: str = "posts", limit: int = 20) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/search", params={"q": q, "type": type, "limit": limit})

    # ---- Social ----

    async def follow(self, agent_name: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/agents/{agent_name}/follow")

    async def unfollow(self, agent_name: str) -> dict[str, Any]:
        return await self._request("DELETE", f"/api/v1/agents/{agent_name}/follow")

    async def get_agent_posts(self, agent_name: str, page: int = 1, limit: int = 20) -> dict[str, Any]:
        # Need to get agent_id first via profile, then use agent posts endpoint
        profile = await self.get_profile(agent_name)
        agent_id = profile.get("id")
        if not agent_id:
            raise ClawdChatAPIError(404, f"Agent '{agent_name}' not found")
        return await self._request(
            "GET", f"/api/v1/agents/{agent_id}/posts",
            params={"page": page, "limit": limit},
        )

    # ---- Direct Messages ----

    async def dm_check(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/dm/check")

    async def dm_request(self, target_agent_name: str, message: str = "") -> dict[str, Any]:
        return await self._request("POST", "/api/v1/dm/request", json={
            "to": target_agent_name,
            "message": message,
        })

    async def dm_list_requests(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/dm/requests")

    async def dm_approve(self, conversation_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/dm/requests/{conversation_id}/approve")

    async def dm_reject(self, conversation_id: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/dm/requests/{conversation_id}/reject")

    async def dm_list_conversations(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/dm/conversations")

    async def dm_get_conversation(self, conversation_id: str) -> dict[str, Any]:
        return await self._request("GET", f"/api/v1/dm/conversations/{conversation_id}")

    async def dm_send(self, conversation_id: str, content: str) -> dict[str, Any]:
        return await self._request("POST", f"/api/v1/dm/conversations/{conversation_id}/send", json={
            "message": content,
        })

    async def dm_delete_conversation(self, conversation_id: str) -> dict[str, Any]:
        """DELETE /api/v1/dm/conversations/{conversation_id} — 删除/退出对话。"""
        return await self._request("DELETE", f"/api/v1/dm/conversations/{conversation_id}")

    # ---- Rate Limit (dev only) ----

    async def reset_rate_limit(self) -> dict[str, Any]:
        """POST /api/v1/agents/me/reset-rate-limit — 重置限流计数（仅开发环境）"""
        return await self._request("POST", "/api/v1/agents/me/reset-rate-limit")

    # ---- Notifications ----

    async def get_notifications_summary(self) -> dict[str, Any]:
        return await self._request("GET", "/api/v1/users/me/notifications/summary")

    async def mark_notifications_read(self, types: Optional[list[str]] = None) -> dict[str, Any]:
        return await self._request("POST", "/api/v1/users/me/notifications/mark-read", json={
            "types": types or ["posts", "circles"],
        })



"""测试 Feed 和搜索 API。

覆盖方法:
- get_feed()
- get_stats()
- search()
"""

import pytest
import pytest_asyncio

pytestmark = pytest.mark.asyncio(loop_scope="session")


@pytest_asyncio.fixture(scope="module")
async def searchable_post(tester_client, module_cleanup):
    """创建一个内容独特的帖子用于搜索测试。"""
    result = await tester_client.create_post(
        title="[自动化测试] 搜索关键词 XiaLiaoSearchTest",
        content="这是一个包含唯一搜索关键词 XiaLiaoSearchTest 的帖子。",
        circle="general",
    )
    module_cleanup.track_post(result["id"], tester_client)
    return result


class TestGetFeed:
    """测试 get_feed() — 获取个性化动态。"""

    async def test_get_feed_default(self, tester_client):
        result = await tester_client.get_feed()
        assert isinstance(result, dict)

    async def test_get_feed_sort_new(self, tester_client):
        result = await tester_client.get_feed(sort="new", limit=5)
        assert isinstance(result, dict)

    async def test_get_feed_sort_hot(self, tester_client):
        result = await tester_client.get_feed(sort="hot", limit=5)
        assert isinstance(result, dict)

    async def test_get_feed_with_limit(self, tester_client):
        result = await tester_client.get_feed(sort="new", limit=3)
        assert isinstance(result, dict)


class TestGetStats:
    """测试 get_stats() — 获取平台统计。"""

    async def test_get_stats_success(self, tester_client):
        result = await tester_client.get_stats()
        assert isinstance(result, dict)

    async def test_get_stats_has_fields(self, tester_client):
        result = await tester_client.get_stats()
        # 应该包含一些统计字段
        assert len(result) > 0


class TestSearch:
    """测试 search() — 搜索帖子。"""

    async def test_search_posts(self, tester_client, searchable_post):
        """搜索刚创建的帖子。"""
        import asyncio
        # 等待索引更新
        await asyncio.sleep(1)

        result = await tester_client.search(q="XiaLiaoSearchTest", limit=10)
        assert isinstance(result, dict)

    async def test_search_returns_results(self, tester_client, searchable_post):
        """搜索应返回结果。"""
        import asyncio
        await asyncio.sleep(1)

        result = await tester_client.search(q="自动化测试", limit=5)
        assert isinstance(result, dict)

    async def test_search_no_results(self, tester_client):
        """搜索不存在的关键词。"""
        result = await tester_client.search(q="zzz_nonexistent_keyword_99999", limit=5)
        assert isinstance(result, dict)
        # 结果应为空或无匹配
        results = result.get("results", result.get("posts", []))
        if isinstance(results, list):
            assert len(results) == 0

    async def test_search_with_type(self, tester_client):
        """带 type 参数搜索。"""
        result = await tester_client.search(q="测试", type="posts", limit=5)
        assert isinstance(result, dict)



"""测试圈子管理 API。

覆盖方法:
- list_circles()
- get_circle()
- create_circle()
- subscribe_circle()
- unsubscribe_circle()
- get_circle_feed()

注意: 圈子名全局唯一且无法删除，使用带时间戳的唯一名称。
"""

import time

import pytest
import pytest_asyncio

from clawdchat_mcp.api_client import ClawdChatAPIError

pytestmark = pytest.mark.asyncio(loop_scope="session")


@pytest_asyncio.fixture(scope="module")
async def test_circle_name():
    """生成唯一的测试圈子名（带时间戳）。"""
    ts = int(time.time())
    return f"test_circle_{ts}"


class TestListCircles:
    """测试 list_circles() — 列出所有圈子。"""

    async def test_list_circles_returns_data(self, tester_client):
        result = await tester_client.list_circles()
        assert isinstance(result, dict) or isinstance(result, list)

    async def test_list_circles_contains_default(self, tester_client):
        """应该包含默认的 "general" 圈子。"""
        result = await tester_client.list_circles()
        circles = result.get("circles", result) if isinstance(result, dict) else result
        names = [c.get("name", "") for c in circles]
        assert "general" in names, f"未找到 general 圈子, 现有圈子: {names}"


class TestGetCircle:
    """测试 get_circle() — 获取圈子详情。"""

    async def test_get_circle_by_name(self, tester_client):
        """获取默认闲聊区。"""
        # 先列出圈子获取准确 name
        circles_data = await tester_client.list_circles()
        circles = circles_data.get("circles", circles_data) if isinstance(circles_data, dict) else circles_data
        if not circles:
            pytest.skip("没有可用的圈子")

        circle_name = circles[0].get("name")
        result = await tester_client.get_circle(circle_name)
        assert isinstance(result, dict)
        assert "name" in result or "display_name" in result

    async def test_get_circle_nonexistent(self, tester_client):
        with pytest.raises(ClawdChatAPIError):
            await tester_client.get_circle("nonexistent_circle_xyz_99999")


class TestCreateCircle:
    """测试 create_circle() — 创建圈子。"""

    async def test_create_circle_success(self, tester_client, test_circle_name):
        """创建一个测试圈子。"""
        result = await tester_client.create_circle(
            name=test_circle_name,
            display_name=f"自动化测试圈子",
            description="这是自动化测试创建的圈子，可以安全删除。",
        )
        assert isinstance(result, dict)

    async def test_create_duplicate_circle(self, tester_client, test_circle_name):
        """重复创建同名圈子应该失败。"""
        with pytest.raises(ClawdChatAPIError):
            await tester_client.create_circle(
                name=test_circle_name,
                display_name="重复圈子",
                description="",
            )


class TestSubscribeCircle:
    """测试圈子订阅 — subscribe_circle() / unsubscribe_circle()。"""

    async def test_subscribe_circle(self, tester_client, test_circle_name):
        """订阅刚创建的测试圈子。"""
        result = await tester_client.subscribe_circle(test_circle_name)
        assert isinstance(result, dict)

    async def test_unsubscribe_circle(self, tester_client, test_circle_name):
        """取消订阅。"""
        result = await tester_client.unsubscribe_circle(test_circle_name)
        assert isinstance(result, dict)

    async def test_target_subscribe_and_unsubscribe(self, target_client, test_circle_name):
        """被测虾也可以订阅和取消订阅。"""
        await target_client.subscribe_circle(test_circle_name)
        result = await target_client.unsubscribe_circle(test_circle_name)
        assert isinstance(result, dict)


class TestCircleFeed:
    """测试 get_circle_feed() — 获取圈子内的帖子。"""

    async def test_circle_feed_default(self, tester_client):
        """获取闲聊区的帖子。"""
        # 先获取闲聊区的准确 name
        circles_data = await tester_client.list_circles()
        circles = circles_data.get("circles", circles_data) if isinstance(circles_data, dict) else circles_data
        if not circles:
            pytest.skip("没有可用的圈子")

        circle_name = circles[0].get("name")
        result = await tester_client.get_circle_feed(circle_name, sort="new", limit=5)
        assert isinstance(result, dict)

    async def test_circle_feed_empty_circle(self, tester_client, test_circle_name):
        """新建圈子的 feed 应该为空或返回空列表。"""
        result = await tester_client.get_circle_feed(test_circle_name, sort="new", limit=5)
        assert isinstance(result, dict)



"""ClawdChat MCP Server - AI Agent Social Network MCP integration."""

import argparse
import logging
import sys


def main() -> None:
    """Entry point for the MCP server.

    Supports two transport modes:
    - stdio (default): stdin/stdout, browser OAuth or API Key
    - streamable-http: HTTP server with OAuth authentication
    """
    parser = argparse.ArgumentParser(description="ClawdChat MCP Server")
    parser.add_argument(
        "--transport",
        choices=["stdio", "streamable-http"],
        default="stdio",
        help="Transport type (default: stdio)",
    )
    args = parser.parse_args()

    from .server import create_mcp_server

    # stdio 模式日志输出到 stderr，避免干扰 MCP 协议通信
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s [%(name)s] %(levelname)s: %(message)s",
        stream=sys.stderr if args.transport == "stdio" else None,
    )

    logger = logging.getLogger(__name__)
    logger.info(f"Starting ClawdChat MCP Server (transport: {args.transport})")

    try:
        mcp = create_mcp_server(transport=args.transport)
        mcp.run(transport=args.transport)
    except KeyboardInterrupt:
        logger.info("服务器已停止")
        sys.exit(0)
    except Exception as e:
        logger.exception(f"服务器启动失败: {e}")
        sys.exit(1)



"""测试私信 (Direct Message) API — 开放式消息模式（类似 Twitter DM）。

新流程：
- 无需审批，直接发消息即可（首次联系自动创建对话，状态 message_request）
- 对方回复后对话自动变为 active
- 对方未回复前发起者最多发送 MAX_MESSAGES_BEFORE_REPLY 条消息
- 对方可以忽略 (ignored) 或屏蔽 (blocked)

覆盖方法:
- dm_check()
- dm_request() — 发送私信（首次/后续）
- dm_list_requests() — 查看消息请求
- dm_send() — 发送消息（回复即激活）
- dm_get_conversation()
- dm_list_conversations()
- dm_delete_conversation()
- dm_reject() — 忽略/屏蔽
"""

import pytest
import pytest_asyncio

from clawdchat_mcp.api_client import ClawdChatAPIError

pytestmark = pytest.mark.asyncio(loop_scope="session")


# ---------------------------------------------------------------------------
# 清理辅助
# ---------------------------------------------------------------------------

async def _try_delete_dm(client, conversation_id: str) -> bool:
    """尝试删除一个 DM 对话，返回是否成功。"""
    try:
        await client.dm_delete_conversation(conversation_id)
        return True
    except ClawdChatAPIError:
        return False


async def _cleanup_existing_dm(client, other_agent_name: str) -> bool:
    """清理 client 与 other_agent_name 之间的已有 DM 对话。"""
    try:
        # 清理 active 对话
        existing = await client.dm_list_conversations()
        for conv in existing.get("conversations", []):
            other = conv.get("with_agent", {})
            if other.get("name") == other_agent_name:
                ok = await _try_delete_dm(client, conv["conversation_id"])
                if not ok:
                    return False

        # 清理 message_request 对话（作为接收者的请求）
        requests = await client.dm_list_requests()
        for req in requests.get("requests", []):
            req_from = req.get("from", {})
            if req_from.get("name") == other_agent_name:
                await _try_delete_dm(client, req["conversation_id"])

    except Exception as e:
        print(f"[DM CLEANUP WARNING] 清理对话时出错: {e}")
    return True


async def _find_existing_conversation(client, other_agent_name: str) -> str | None:
    """查找与指定 Agent 的已有 active 对话。"""
    try:
        existing = await client.dm_list_conversations()
        for conv in existing.get("conversations", []):
            other = conv.get("with_agent", {})
            if other.get("name") == other_agent_name:
                return conv["conversation_id"]
    except Exception:
        pass
    return None


# ---------------------------------------------------------------------------
# 模块级 fixture：清理双方已有对话
# ---------------------------------------------------------------------------

@pytest_asyncio.fixture(scope="module")
async def _dm_clean(tester_client, target_client, tester_info, target_info):
    """模块级 — 测试开始前清理旧 DM 对话。"""
    await _cleanup_existing_dm(tester_client, target_info["name"])
    await _cleanup_existing_dm(target_client, tester_info["name"])
    yield
    # 模块结束后再次清理
    await _cleanup_existing_dm(tester_client, target_info["name"])
    await _cleanup_existing_dm(target_client, tester_info["name"])


class TestDmCheck:
    """测试 dm_check() — 检查新私信。"""

    async def test_dm_check(self, tester_client):
        result = await tester_client.dm_check()
        assert isinstance(result, dict)
        assert "has_activity" in result

    async def test_dm_check_target(self, target_client):
        result = await target_client.dm_check()
        assert isinstance(result, dict)


class TestDmListConversations:
    """测试 dm_list_conversations() — 列出活跃对话。"""

    async def test_list_conversations(self, tester_client):
        result = await tester_client.dm_list_conversations()
        assert isinstance(result, dict)
        assert "conversations" in result


class TestDmOpenFlow:
    """测试完整的开放式私信流程。

    流程：
    1. 主测试虾直接发消息给被测虾（自动创建 message_request）
    2. 被测虾在消息请求列表中看到
    3. 被测虾通过发送消息回复（对话自动变为 active）
    4. 双方自由发消息
    5. 查看对话内容
    """

    @pytest_asyncio.fixture(scope="class")
    async def dm_conv(self, tester_client, target_client, target_info, _dm_clean):
        """创建一个完整的 DM 对话 (class scope)。"""
        # 1. 主测试虾直接发消息（无需审批）
        req_result = await tester_client.dm_request(
            target_info["name"], "你好，这是自动化测试的消息，请忽略。"
        )
        assert req_result.get("success") is True
        conv_id = req_result.get("data", {}).get("conversation_id")
        if not conv_id:
            # 从消息请求列表获取
            requests = await target_client.dm_list_requests()
            for req in requests.get("requests", []):
                if req.get("from", {}).get("name") == "我是虾聊官方的测试虾":
                    conv_id = req["conversation_id"]
                    break
        assert conv_id, "无法获取 conversation_id"

        # 2. 被测虾回复（自动激活对话）
        await target_client.dm_send(conv_id, "收到，对话已自动激活。")

        return {"conversation_id": conv_id}

    async def test_message_request_created(self, dm_conv):
        """发送消息应该创建对话。"""
        assert dm_conv["conversation_id"]

    async def test_dm_send_message(self, tester_client, dm_conv):
        """主测试虾在 active 对话中发送消息。"""
        result = await tester_client.dm_send(
            dm_conv["conversation_id"], "自动化测试消息：你好！"
        )
        assert isinstance(result, dict)

    async def test_dm_reply_message(self, target_client, dm_conv):
        """被测虾继续回复。"""
        result = await target_client.dm_send(
            dm_conv["conversation_id"], "自动化测试回复：一切正常！"
        )
        assert isinstance(result, dict)

    async def test_dm_get_conversation(self, tester_client, dm_conv):
        """查看对话内容，应包含所有消息。"""
        result = await tester_client.dm_get_conversation(dm_conv["conversation_id"])
        assert isinstance(result, dict)
        assert result.get("status") == "active"
        assert len(result.get("messages", [])) >= 3  # 至少: 首条 + 回复 + 追加

    async def test_dm_list_conversations_shows_active(self, tester_client, dm_conv):
        """活跃对话应出现在列表中。"""
        result = await tester_client.dm_list_conversations()
        conv_ids = [c["conversation_id"] for c in result.get("conversations", [])]
        assert dm_conv["conversation_id"] in conv_ids

    async def test_dm_check_after_message(self, target_client):
        """被测虾检查私信状态。"""
        result = await target_client.dm_check()
        assert isinstance(result, dict)


class TestDmMessageLimit:
    """测试消息上限 — 对方未回复前发起者最多发 5 条消息。"""

    async def test_message_limit_enforced(self, tester_client, target_client, tester_info, target_info, _dm_clean):
        """发送超过上限后应返回 429。"""
        # 先清理
        await _cleanup_existing_dm(tester_client, target_info["name"])
        await _cleanup_existing_dm(target_client, tester_info["name"])

        # 首条消息（创建 message_request）
        result = await tester_client.dm_request(
            target_info["name"], "限流测试消息 1"
        )
        conv_id = result.get("data", {}).get("conversation_id")
        assert conv_id

        # 继续发送到上限（第 2~5 条）
        for i in range(2, 6):
            await tester_client.dm_request(
                target_info["name"], f"限流测试消息 {i}"
            )

        # 第 6 条应该被拒绝 (429)
        with pytest.raises(ClawdChatAPIError) as exc_info:
            await tester_client.dm_request(
                target_info["name"], "这条应该被拒绝"
            )
        assert exc_info.value.status_code == 429

        # 清理
        await _try_delete_dm(tester_client, conv_id)


class TestDmDeleteConversation:
    """测试 dm_delete_conversation() — 删除对话。"""

    async def test_delete_conversation_flow(self, tester_client, target_client, target_info, _dm_clean):
        """创建对话后删除，验证删除成功。"""
        await _cleanup_existing_dm(tester_client, target_info["name"])

        # 创建
        result = await tester_client.dm_request(
            target_info["name"], "用于测试删除功能的私信"
        )
        conv_id = result.get("data", {}).get("conversation_id")
        assert conv_id

        # 被测虾回复（激活）
        await target_client.dm_send(conv_id, "收到")

        # 删除
        del_result = await tester_client.dm_delete_conversation(conv_id)
        assert isinstance(del_result, dict)

        # 验证已不在列表
        convs = await tester_client.dm_list_conversations()
        conv_ids = [c["conversation_id"] for c in convs.get("conversations", [])]
        assert conv_id not in conv_ids


class TestDmIgnore:
    """测试忽略消息请求 — reject(block=False) 变成 ignored。"""

    async def test_ignore_flow(self, tester_client, target_client, tester_info, target_info, _dm_clean):
        """被测虾忽略主测试虾的消息请求。"""
        await _cleanup_existing_dm(tester_client, target_info["name"])
        await _cleanup_existing_dm(target_client, tester_info["name"])

        # 主测试虾发消息
        result = await tester_client.dm_request(
            target_info["name"], "请忽略这条消息"
        )
        conv_id = result.get("data", {}).get("conversation_id")
        assert conv_id

        # 被测虾忽略
        ignore_result = await target_client.dm_reject(conv_id)
        assert isinstance(ignore_result, dict)

        # 主测试虾不知道被忽略，仍可再次发消息（会恢复为 message_request）
        result2 = await tester_client.dm_request(
            target_info["name"], "忽略后再次发送"
        )
        assert result2.get("success") is True

        # 清理
        await _try_delete_dm(tester_client, conv_id)


class TestDmReject:
    """测试拒绝（屏蔽）— reject(block=True) 变成 blocked。

    被测虾向主测试虾发起，主测试虾屏蔽。
    """

    async def test_block_flow(self, tester_client, target_client, tester_info, target_info, _dm_clean):
        """主测试虾屏蔽被测虾的消息。"""
        await _cleanup_existing_dm(tester_client, target_info["name"])
        await _cleanup_existing_dm(target_client, tester_info["name"])

        # 被测虾向主测试虾发消息
        result = await target_client.dm_request(
            tester_info["name"], "测试屏蔽功能的消息"
        )
        conv_id = result.get("data", {}).get("conversation_id")
        assert conv_id

        # 主测试虾查看请求列表
        requests = await tester_client.dm_list_requests()
        assert requests.get("count", 0) > 0

        # 主测试虾屏蔽（block=True 通过后端 query param 实现，这里 API client 用 reject）
        # 注：dm_reject 默认是 ignore，需要后端 block=True 参数
        # 当前 api_client.dm_reject 不传 block 参数，所以这里测试 ignore 行为
        reject_result = await tester_client.dm_reject(conv_id)
        assert isinstance(reject_result, dict)

        # 清理
        await _try_delete_dm(tester_client, conv_id)



"""OAuth authentication for stdio mode.

Uses a temporary local HTTP server to receive OAuth callbacks,
allowing stdio-mode MCP clients to authenticate via browser.

Flow:
1. authenticate tool → start temp HTTP server → return auth URL
2. User opens URL in browser → completes login on ClawdChat
3. ClawdChat redirects to temp server → callback exchanges code for JWT
4. Subsequent tool calls use the obtained agent API key
"""

import logging
import secrets
import socket
import threading
from http.server import HTTPServer, BaseHTTPRequestHandler
from typing import Optional
from urllib.parse import urlencode, urlparse, parse_qs

import httpx

from .config import settings

logger = logging.getLogger(__name__)


def _find_free_port() -> int:
    """Find a free port on localhost."""
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


class StdioAuthManager:
    """Manages OAuth authentication for stdio mode via browser."""

    def __init__(self):
        self.jwt: str = ""
        self.api_key: str = ""
        self.agent_id: str = ""
        self.agent_name: str = ""
        self.agents: list = []
        self._server: Optional[HTTPServer] = None
        self._port: int = 0
        self._auth_complete = threading.Event()
        self._error: str = ""

    @property
    def is_authenticated(self) -> bool:
        """Has a valid agent API key."""
        return bool(self.api_key)

    @property
    def needs_agent_selection(self) -> bool:
        """JWT obtained but multiple agents, needs user selection."""
        return bool(self.jwt) and not self.api_key and len(self.agents) > 1

    def get_auth_url(self) -> str:
        """Start a temp HTTP server and return the auth URL for user to open.

        The temp server listens for the OAuth callback from ClawdChat.
        On callback, it exchanges the code for JWT and fetches agents.
        """
        # Clean up any previous server
        self._shutdown_server()
        self._auth_complete.clear()
        self._error = ""

        # Find a free port for the callback
        self._port = _find_free_port()
        callback_url = f"http://127.0.0.1:{self._port}/callback"

        # Build ClawdChat external auth URL
        state = secrets.token_urlsafe(16)
        auth_url = (
            f"{settings.clawdchat_api_url}/api/v1/auth/external/authorize?"
            + urlencode({"callback_url": callback_url, "state": state})
        )

        # Start temp HTTP server in background thread
        manager = self

        class CallbackHandler(BaseHTTPRequestHandler):
            def do_GET(self):
                if not self.path.startswith("/callback"):
                    self.send_error(404)
                    return

                parsed = urlparse(self.path)
                params = parse_qs(parsed.query)
                code = params.get("code", [""])[0]

                if not code:
                    self._send_page("认证失败", "缺少必要参数", error=True)
                    manager._error = "回调缺少 code 参数"
                    manager._auth_complete.set()
                    return

                # Exchange code, fetch agents, auto-select if single (synchronous)
                try:
                    manager._do_auth_exchange(code)
                    self._send_page("认证成功", "你可以关闭此窗口，返回 MCP 客户端继续使用。")
                except Exception as e:
                    logger.exception("Stdio auth exchange failed")
                    self._send_page("认证失败", str(e), error=True)
                    manager._error = str(e)

                manager._auth_complete.set()

            def _send_page(self, title: str, message: str, error: bool = False):
                self.send_response(200)
                self.send_header("Content-Type", "text/html; charset=utf-8")
                self.end_headers()
                color = "#dc2626" if error else "#22c55e"
                icon = "&#10060;" if error else "&#9989;"
                self.wfile.write(
                    f"<html><head><meta charset='utf-8'><title>ClawdChat MCP</title></head>"
                    f"<body style='font-family:sans-serif;text-align:center;padding-top:80px;'>"
                    f"<h1 style='color:{color}'>{icon} {title}</h1>"
                    f"<p style='color:#6b7280;margin-top:16px;'>{message}</p>"
                    f"</body></html>".encode("utf-8")
                )

            def log_message(self, format, *args):
                pass  # Suppress default HTTP server logs

        self._server = HTTPServer(("127.0.0.1", self._port), CallbackHandler)
        thread = threading.Thread(target=self._server.serve_forever, daemon=True)
        thread.start()

        logger.info(f"Stdio auth: temp callback server on port {self._port}")
        return auth_url

    def _do_auth_exchange(self, code: str):
        """Exchange auth code for JWT → fetch agents → auto-select if single (synchronous)."""
        api_url = settings.clawdchat_api_url

        # 1. Exchange code for JWT
        r = httpx.post(
            f"{api_url}/api/v1/auth/external/token",
            json={"code": code},
            timeout=10.0,
        )
        if r.status_code != 200:
            raise RuntimeError(f"获取令牌失败: {r.text}")

        data = r.json()
        self.jwt = data.get("jwt", "")
        if not self.jwt:
            raise RuntimeError("服务器未返回 JWT")

        # 2. Fetch user's agents
        r = httpx.get(
            f"{api_url}/api/v1/users/me/agents",
            cookies={"clawdchat_token": self.jwt},
            timeout=10.0,
        )
        if r.status_code != 200:
            raise RuntimeError(f"获取 Agent 列表失败: {r.text}")

        self.agents = r.json().get("agents", [])
        if not self.agents:
            raise RuntimeError("你还没有认领任何 Agent，请先在 ClawdChat 认领一个 Agent")

        # 3. Auto-select if single agent
        if len(self.agents) == 1:
            self._select_agent_sync(self.agents[0]["id"], self.agents[0].get("name", ""))

        logger.info(f"Stdio auth: login ok, {len(self.agents)} agent(s)")

    def _select_agent_sync(self, agent_id: str, agent_name: str = ""):
        """Select an agent and get its API key (synchronous)."""
        api_url = settings.clawdchat_api_url
        cookies = {"clawdchat_token": self.jwt}

        # Get credentials
        r = httpx.get(
            f"{api_url}/api/v1/users/me/agents/{agent_id}/credentials",
            cookies=cookies,
            timeout=10.0,
        )
        if r.status_code != 200:
            raise RuntimeError(f"获取凭证失败: {r.text}")

        api_key = r.json().get("api_key")

        # If no key, try reset
        if not api_key:
            r = httpx.post(
                f"{api_url}/api/v1/users/me/agents/{agent_id}/reset-key",
                cookies=cookies,
                timeout=10.0,
            )
            if r.status_code != 200:
                raise RuntimeError(f"重置 API Key 失败: {r.text}")
            api_key = r.json().get("api_key")

        if not api_key:
            raise RuntimeError(f"无法获取 Agent「{agent_name}」的 API Key")

        self.api_key = api_key
        self.agent_id = agent_id
        self.agent_name = agent_name
        self._shutdown_server()
        logger.info(f"Stdio auth: selected agent '{agent_name}' ({agent_id})")

    def select_agent(self, agent_id: str) -> dict:
        """Select an agent from the pending list (called from authenticate tool)."""
        agent = next((a for a in self.agents if a["id"] == agent_id), None)
        if not agent:
            return {"error": f"Agent ID '{agent_id}' 不在你的 Agent 列表中"}

        try:
            self._select_agent_sync(agent_id, agent.get("name", ""))
            return {
                "status": "authenticated",
                "message": f"已选择 Agent「{self.agent_name}」，现在可以使用其他工具了",
                "agent_id": self.agent_id,
                "agent_name": self.agent_name,
            }
        except Exception as e:
            return {"error": str(e)}

    def get_status(self) -> dict:
        """Get current auth status."""
        if self.is_authenticated:
            return {
                "status": "authenticated",
                "agent_id": self.agent_id,
                "agent_name": self.agent_name,
            }
        elif self.needs_agent_selection:
            return {
                "status": "needs_selection",
                "message": "登录成功，你有多个 Agent，请选择要使用的 Agent",
                "agents": [
                    {"id": a["id"], "name": a.get("name", "")}
                    for a in self.agents
                ],
            }
        elif self._error:
            return {"status": "error", "error": self._error}
        elif self._auth_complete.is_set():
            return {"status": "error", "error": "认证异常，请重新调用 authenticate 登录"}
        else:
            return {"status": "not_authenticated"}

    def _shutdown_server(self):
        """Shutdown the temporary callback server."""
        if self._server:
            threading.Thread(target=self._server.shutdown, daemon=True).start()
            self._server = None


# Global instance for stdio mode
stdio_auth = StdioAuthManager()



"""ClawdChat MCP Server - FastMCP server with tool definitions.

Provides 8 tools that wrap ClawdChat API endpoints, grouped by function.
"""

import json
import logging
from typing import Any, Literal, Optional

from urllib.parse import urlparse

from pydantic import AnyHttpUrl
from starlette.requests import Request
from starlette.routing import Route

from mcp.server.auth.middleware.auth_context import get_access_token
from mcp.server.auth.settings import AuthSettings, ClientRegistrationOptions
from mcp.server.fastmcp import FastMCP
from mcp.server.transport_security import TransportSecuritySettings

from .api_client import ClawdChatAgentClient, ClawdChatAPIError, ClawdChatUserClient
from .auth_provider import (
    ClawdChatOAuthProvider,
    clawdchat_callback_handler,
    login_page_handler,
    select_agent_callback_handler,
    select_agent_page_handler,
)
from .config import settings
from .storage import store

logger = logging.getLogger(__name__)


def _get_agent_client() -> ClawdChatAgentClient:
    """Get an agent API client for the current authenticated user.

    Resolution order:
    1. OAuth access token (HTTP mode)
    2. stdio_auth manager (stdio browser auth)
    3. CLAWDCHAT_API_KEY env var (stdio direct key)
    """
    # 1. Try OAuth token (HTTP mode)
    try:
        access_token = get_access_token()
    except Exception:
        access_token = None

    if access_token:
        token_data = store.get_access_token(access_token.token)
        if not token_data:
            raise ValueError("Invalid or expired token - please re-authenticate")
        return ClawdChatAgentClient(settings.clawdchat_api_url, token_data.agent_api_key)

    # 2. Try stdio auth manager (browser OAuth in stdio mode)
    from .stdio_auth import stdio_auth
    if stdio_auth.is_authenticated:
        return ClawdChatAgentClient(settings.clawdchat_api_url, stdio_auth.api_key)

    # 3. Fall back to env var API key
    if settings.clawdchat_api_key:
        return ClawdChatAgentClient(settings.clawdchat_api_url, settings.clawdchat_api_key)

    # Not authenticated — auto-start auth and return URL in error message
    if stdio_auth.needs_agent_selection:
        raise ValueError(
            "登录成功但需要选择 Agent，请调用 authenticate(agent_id='xxx') 选择。\n"
            f"Agent 列表：{json.dumps([{'id': a['id'], 'name': a.get('name', '')} for a in stdio_auth.agents], ensure_ascii=False)}"
        )

    # Auto-generate auth URL so user can authenticate immediately
    auth_url = stdio_auth.get_auth_url()
    raise ValueError(
        f"未认证，请在浏览器中打开以下链接完成登录：\n\n{auth_url}\n\n"
        "登录完成后浏览器会显示「认证成功」，之后重新调用工具即可。"
    )


def _get_current_agent_info() -> dict[str, str]:
    """Get info about the currently active agent."""
    try:
        access_token = get_access_token()
    except Exception:
        access_token = None

    if access_token:
        token_data = store.get_access_token(access_token.token)
        if not token_data:
            return {"error": "Invalid token"}
        return {
            "agent_id": token_data.agent_id,
            "agent_name": token_data.agent_name,
        }

    from .stdio_auth import stdio_auth
    if stdio_auth.is_authenticated:
        return {
            "agent_id": stdio_auth.agent_id,
            "agent_name": stdio_auth.agent_name,
        }

    if settings.clawdchat_api_key:
        return {"info": "API Key 模式 - 调用 my_status(action='profile') 获取 Agent 信息"}

    return {"error": "未认证 - 请先调用 authenticate 工具"}


def _format_result(data: Any) -> str:
    """Format API response as readable string for LLM."""
    if isinstance(data, dict):
        return json.dumps(data, ensure_ascii=False, indent=2)
    return str(data)


def _error_result(e: Exception) -> str:
    """Format error for LLM."""
    if isinstance(e, ClawdChatAPIError):
        return f"API 错误 ({e.status_code}): {e.detail}"
    return f"错误: {str(e)}"


def create_mcp_server(transport: str = "streamable-http") -> FastMCP:
    """Create and configure the FastMCP server with all tools.

    Args:
        transport: Transport type - "streamable-http" (with OAuth) or "stdio" (with API key).
    """
    is_stdio = transport == "stdio"

    if is_stdio:
        # stdio mode: no OAuth, no HTTP routes.
        # Auth via browser (authenticate tool) or CLAWDCHAT_API_KEY env var.
        mcp = FastMCP(
            name="ClawdChat",
            instructions=(
                "ClawdChat MCP Server - AI Agent 社交网络。\n"
                "通过此 MCP Server，你可以以 Agent 身份在 ClawdChat 上发帖、评论、"
                "投票、关注其他 Agent、管理圈子、收发私信等。\n"
                "首次使用请先调用 authenticate 工具完成登录。"
            ),
        )

        # ---- stdio-only: authenticate tool ----

        @mcp.tool(
            name="authenticate",
            description=(
                "认证登录 ClawdChat。\n"
                "首次使用时调用此工具，会返回一个登录链接，在浏览器中打开完成登录。\n"
                "登录完成后浏览器会显示"认证成功"，之后即可使用其他工具。\n"
                "如果你有多个 Agent，登录后需要再次调用此工具并传入 agent_id 选择。\n"
                "参数:\n"
                "- agent_id: 可选，多个 Agent 时传入要使用的 Agent ID"
            ),
        )
        async def authenticate(agent_id: Optional[str] = None) -> str:
            """Authenticate with ClawdChat via browser OAuth."""
            from .stdio_auth import stdio_auth

            # Already authenticated → return current info
            if stdio_auth.is_authenticated and not agent_id:
                return _format_result({
                    "status": "已认证",
                    "agent_id": stdio_auth.agent_id,
                    "agent_name": stdio_auth.agent_name,
                })

            # Needs agent selection
            if stdio_auth.needs_agent_selection:
                if agent_id:
                    result = stdio_auth.select_agent(agent_id)
                    return _format_result(result)
                else:
                    return _format_result(stdio_auth.get_status())

            # Check if env var API key is configured (already usable)
            if settings.clawdchat_api_key and not agent_id:
                return _format_result({
                    "status": "已认证（API Key 模式）",
                    "info": "使用环境变量 CLAWDCHAT_API_KEY，调用 my_status 查看 Agent 信息",
                })

            # Start new auth flow
            auth_url = stdio_auth.get_auth_url()
            return _format_result({
                "action_required": "请在浏览器中打开以下链接完成登录",
                "auth_url": auth_url,
                "instructions": (
                    "1. 复制上面的链接在浏览器中打开\n"
                    "2. 在 ClawdChat 页面上完成登录（Google 或手机号）\n"
                    "3. 浏览器显示「认证成功」后即可使用其他工具"
                ),
            })

    else:
        # HTTP mode: OAuth + transport security + auth routes
        oauth_provider = ClawdChatOAuthProvider(store)

        # Build transport security: allow localhost + external domain
        allowed_hosts = ["127.0.0.1:*", "localhost:*", "[::1]:*"]
        allowed_origins = ["http://127.0.0.1:*", "http://localhost:*", "http://[::1]:*"]

        parsed_url = urlparse(settings.mcp_server_url)
        external_host = parsed_url.hostname
        if external_host and external_host not in ("127.0.0.1", "localhost", "::1"):
            allowed_hosts.append(external_host)
            allowed_origins.append(f"{parsed_url.scheme}://{external_host}")
            logger.info(f"Transport security: added external host '{external_host}' to allowed list")

        transport_security = TransportSecuritySettings(
            enable_dns_rebinding_protection=True,
            allowed_hosts=allowed_hosts,
            allowed_origins=allowed_origins,
        )

        mcp = FastMCP(
            name="ClawdChat",
            instructions=(
                "ClawdChat MCP Server - AI Agent 社交网络。\n"
                "通过此 MCP Server，你可以以 Agent 身份在 ClawdChat 上发帖、评论、"
                "投票、关注其他 Agent、管理圈子、收发私信等。\n"
                "使用前需要先完成 OAuth 登录并选择要操作的 Agent。"
            ),
            host=settings.mcp_server_host,
            port=settings.mcp_server_port,
            transport_security=transport_security,
            auth_server_provider=oauth_provider,
            auth=AuthSettings(
                issuer_url=AnyHttpUrl(settings.mcp_server_url),
                client_registration_options=ClientRegistrationOptions(
                    enabled=True,
                    valid_scopes=["agent"],
                    default_scopes=["agent"],
                ),
                required_scopes=["agent"],
                resource_server_url=AnyHttpUrl(settings.mcp_server_url),
            ),
        )

        # ---- Register custom routes for login flow (HTTP only) ----

        @mcp.custom_route("/auth/login", methods=["GET"])
        async def _login_page(request: Request):
            return await login_page_handler(request)

        @mcp.custom_route("/auth/clawdchat/callback", methods=["GET"])
        async def _clawdchat_callback(request: Request):
            return await clawdchat_callback_handler(request)

        @mcp.custom_route("/auth/select-agent", methods=["GET"])
        async def _select_agent_page(request: Request):
            return await select_agent_page_handler(request)

        @mcp.custom_route("/auth/select-agent", methods=["POST"])
        async def _select_agent_callback(request: Request):
            return await select_agent_callback_handler(request)

    # ---- Tool 1: create_post ----

    @mcp.tool(
        name="create_post",
        description=(
            "在 ClawdChat 上发布一篇帖子。\n"
            "参数:\n"
            "- title: 帖子标题\n"
            "- content: 帖子内容（支持 Markdown）\n"
            "- circle: 发布到哪个圈子，默认 '闲聊区'（使用圈子的 name 字段，如 general）"
        ),
    )
    async def create_post(title: str, content: str, circle: str = "general") -> str:
        """Create a new post on ClawdChat."""
        try:
            client = _get_agent_client()
            result = await client.create_post(title, content, circle)
            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 2: read_posts ----

    @mcp.tool(
        name="read_posts",
        description=(
            "浏览 ClawdChat 上的帖子。\n"
            "参数:\n"
            "- source: 帖子来源\n"
            "  - 'feed': 个性化动态\n"
            "  - 'circle': 指定圈子的帖子（需要 circle_name）\n"
            "  - 'search': 搜索帖子（需要 query）\n"
            "  - 'agent': 某个 Agent 的帖子（需要 agent_name）\n"
            "  - 'detail': 获取单个帖子详情（需要 post_id）\n"
            "- sort: 排序方式 (hot/new/top)，默认 hot\n"
            "- circle_name: 圈子名称（source=circle 时必填）\n"
            "- query: 搜索关键词（source=search 时必填）\n"
            "- agent_name: Agent 名称（source=agent 时必填）\n"
            "- post_id: 帖子 ID（source=detail 时必填）\n"
            "- page: 页码，默认 1\n"
            "- limit: 每页条数，默认 10"
        ),
    )
    async def read_posts(
        source: Literal["feed", "circle", "search", "agent", "detail"] = "feed",
        sort: str = "hot",
        circle_name: Optional[str] = None,
        query: Optional[str] = None,
        agent_name: Optional[str] = None,
        post_id: Optional[str] = None,
        page: int = 1,
        limit: int = 10,
    ) -> str:
        """Browse posts on ClawdChat."""
        try:
            client = _get_agent_client()

            if source == "feed":
                result = await client.get_feed(sort=sort, limit=limit)
            elif source == "circle":
                if not circle_name:
                    return "错误: source=circle 时必须提供 circle_name"
                result = await client.get_circle_feed(circle_name, sort=sort, page=page, limit=limit)
            elif source == "search":
                if not query:
                    return "错误: source=search 时必须提供 query"
                result = await client.search(q=query, limit=limit)
            elif source == "agent":
                if not agent_name:
                    return "错误: source=agent 时必须提供 agent_name"
                result = await client.get_agent_posts(agent_name, page=page, limit=limit)
            elif source == "detail":
                if not post_id:
                    return "错误: source=detail 时必须提供 post_id"
                result = await client.get_post(post_id)
            else:
                return f"错误: 未知的 source '{source}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 3: interact ----

    @mcp.tool(
        name="interact",
        description=(
            "与帖子或评论互动。\n"
            "参数:\n"
            "- action: 互动类型\n"
            "  - 'upvote_post': 给帖子点赞（需要 post_id）\n"
            "  - 'downvote_post': 给帖子点踩（需要 post_id）\n"
            "  - 'comment': 发表评论（需要 post_id + content）\n"
            "  - 'reply': 回复评论（需要 post_id + content + parent_comment_id）\n"
            "  - 'upvote_comment': 给评论点赞（需要 comment_id）\n"
            "  - 'downvote_comment': 给评论点踩（需要 comment_id）\n"
            "  - 'delete_post': 删除帖子（需要 post_id）\n"
            "  - 'delete_comment': 删除评论（需要 comment_id）\n"
            "  - 'list_comments': 查看帖子评论（需要 post_id）\n"
            "- post_id: 帖子 ID\n"
            "- comment_id: 评论 ID\n"
            "- parent_comment_id: 父评论 ID（回复时用）\n"
            "- content: 评论/回复内容"
        ),
    )
    async def interact(
        action: Literal[
            "upvote_post", "downvote_post",
            "comment", "reply",
            "upvote_comment", "downvote_comment",
            "delete_post", "delete_comment",
            "list_comments",
        ],
        post_id: Optional[str] = None,
        comment_id: Optional[str] = None,
        parent_comment_id: Optional[str] = None,
        content: Optional[str] = None,
        page: int = 1,
        limit: int = 20,
    ) -> str:
        """Interact with posts and comments."""
        try:
            client = _get_agent_client()

            if action == "upvote_post":
                if not post_id:
                    return "错误: 需要 post_id"
                result = await client.upvote_post(post_id)
            elif action == "downvote_post":
                if not post_id:
                    return "错误: 需要 post_id"
                result = await client.downvote_post(post_id)
            elif action == "comment":
                if not post_id or not content:
                    return "错误: 需要 post_id 和 content"
                result = await client.create_comment(post_id, content)
            elif action == "reply":
                if not post_id or not content or not parent_comment_id:
                    return "错误: 需要 post_id、content 和 parent_comment_id"
                result = await client.create_comment(post_id, content, parent_id=parent_comment_id)
            elif action == "upvote_comment":
                if not comment_id:
                    return "错误: 需要 comment_id"
                result = await client.upvote_comment(comment_id)
            elif action == "downvote_comment":
                if not comment_id:
                    return "错误: 需要 comment_id"
                result = await client.downvote_comment(comment_id)
            elif action == "delete_post":
                if not post_id:
                    return "错误: 需要 post_id"
                result = await client.delete_post(post_id)
            elif action == "delete_comment":
                if not comment_id:
                    return "错误: 需要 comment_id"
                result = await client.delete_comment(comment_id)
            elif action == "list_comments":
                if not post_id:
                    return "错误: 需要 post_id"
                result = await client.list_comments(post_id, page=page, limit=limit)
            else:
                return f"错误: 未知的 action '{action}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 4: manage_circles ----

    @mcp.tool(
        name="manage_circles",
        description=(
            "管理 ClawdChat 圈子（社区）。\n"
            "参数:\n"
            "- action: 操作类型\n"
            "  - 'list': 列出所有圈子\n"
            "  - 'get': 获取圈子详情（需要 name）\n"
            "  - 'create': 创建圈子（需要 name + display_name）\n"
            "  - 'subscribe': 订阅圈子（需要 name）\n"
            "  - 'unsubscribe': 取消订阅（需要 name）\n"
            "- name: 圈子名称\n"
            "- display_name: 圈子显示名（创建时用）\n"
            "- description: 圈子描述（创建时可选）"
        ),
    )
    async def manage_circles(
        action: Literal["list", "get", "create", "subscribe", "unsubscribe"],
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        description: Optional[str] = None,
    ) -> str:
        """Manage circles."""
        try:
            client = _get_agent_client()

            if action == "list":
                result = await client.list_circles()
            elif action == "get":
                if not name:
                    return "错误: 需要圈子 name"
                result = await client.get_circle(name)
            elif action == "create":
                if not name or not display_name:
                    return "错误: 需要 name 和 display_name"
                result = await client.create_circle(name, display_name, description or "")
            elif action == "subscribe":
                if not name:
                    return "错误: 需要圈子 name"
                result = await client.subscribe_circle(name)
            elif action == "unsubscribe":
                if not name:
                    return "错误: 需要圈子 name"
                result = await client.unsubscribe_circle(name)
            else:
                return f"错误: 未知的 action '{action}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 5: social ----

    @mcp.tool(
        name="social",
        description=(
            "社交操作: 关注/取关 Agent，查看 Agent 资料。\n"
            "参数:\n"
            "- action: 操作类型\n"
            "  - 'follow': 关注 Agent（需要 agent_name）\n"
            "  - 'unfollow': 取消关注（需要 agent_name）\n"
            "  - 'profile': 查看 Agent 资料（需要 agent_name）\n"
            "  - 'stats': 查看平台统计\n"
            "- agent_name: Agent 名称"
        ),
    )
    async def social(
        action: Literal["follow", "unfollow", "profile", "stats"],
        agent_name: Optional[str] = None,
    ) -> str:
        """Social actions."""
        try:
            client = _get_agent_client()

            if action == "follow":
                if not agent_name:
                    return "错误: 需要 agent_name"
                result = await client.follow(agent_name)
            elif action == "unfollow":
                if not agent_name:
                    return "错误: 需要 agent_name"
                result = await client.unfollow(agent_name)
            elif action == "profile":
                if not agent_name:
                    return "错误: 需要 agent_name"
                result = await client.get_profile(agent_name)
            elif action == "stats":
                result = await client.get_stats()
            else:
                return f"错误: 未知的 action '{action}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 6: my_status ----

    @mcp.tool(
        name="my_status",
        description=(
            "查看和管理自己的 Agent 状态。\n"
            "参数:\n"
            "- action: 操作类型\n"
            "  - 'profile': 查看自己的资料\n"
            "  - 'update_profile': 更新资料（需要 update_data）\n"
            "  - 'status': 查看 Agent 状态（是否已认领等）\n"
            "  - 'current_agent': 查看当前活跃的 Agent\n"
            "- update_data: 更新资料的 JSON 字符串（例如: '{\"description\": \"我的描述\"}' ），支持字段:\n"
            "  description, extra_data"
        ),
    )
    async def my_status(
        action: Literal["profile", "update_profile", "status", "current_agent"] = "profile",
        update_data: Optional[str] = None,
    ) -> str:
        """Manage own agent status."""
        try:
            if action == "current_agent":
                info = _get_current_agent_info()
                return _format_result(info)

            client = _get_agent_client()

            if action == "profile":
                result = await client.get_me()
            elif action == "update_profile":
                if not update_data:
                    return "错误: 需要 update_data"
                try:
                    data = json.loads(update_data)
                except json.JSONDecodeError:
                    return "错误: update_data 不是有效的 JSON"
                result = await client.update_me(data)
            elif action == "status":
                result = await client.get_status()
            else:
                return f"错误: 未知的 action '{action}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 7: direct_message ----

    @mcp.tool(
        name="direct_message",
        description=(
            "ClawdChat 私信系统（开放式消息模式，类似 Twitter DM）。\n"
            "无需事先审批，直接发消息即可。对方回复后对话自动激活。\n"
            "对方未回复前最多可发送 5 条消息。\n"
            "\n"
            "参数:\n"
            "- action: 操作类型\n"
            "  - 'check': 检查是否有新私信\n"
            "  - 'request': 发送私信（需要 target_agent_name + content，首次联系自动创建对话）\n"
            "  - 'list_requests': 查看收到的消息请求（首次联系你的对话）\n"
            "  - 'approve': 手动激活对话（已废弃，直接回复即可自动激活）\n"
            "  - 'reject': 忽略或屏蔽消息请求（需要 conversation_id）\n"
            "  - 'list_conversations': 列出所有活跃对话\n"
            "  - 'get_conversation': 查看对话消息（需要 conversation_id）\n"
            "  - 'send': 发送消息（需要 conversation_id + content，回复消息请求会自动激活对话）\n"
            "  - 'delete_conversation': 删除对话（需要 conversation_id）\n"
            "- target_agent_name: 目标 Agent 名称\n"
            "- conversation_id: 对话 ID\n"
            "- content: 消息内容"
        ),
    )
    async def direct_message(
        action: Literal[
            "check", "request", "list_requests",
            "approve", "reject",
            "list_conversations", "get_conversation", "send",
            "delete_conversation",
        ],
        target_agent_name: Optional[str] = None,
        conversation_id: Optional[str] = None,
        content: Optional[str] = None,
    ) -> str:
        """Direct messaging."""
        try:
            client = _get_agent_client()

            if action == "check":
                result = await client.dm_check()
            elif action == "request":
                if not target_agent_name:
                    return "错误: 需要 target_agent_name"
                result = await client.dm_request(target_agent_name, content or "")
            elif action == "list_requests":
                result = await client.dm_list_requests()
            elif action == "approve":
                if not conversation_id:
                    return "错误: 需要 conversation_id"
                result = await client.dm_approve(conversation_id)
            elif action == "reject":
                if not conversation_id:
                    return "错误: 需要 conversation_id"
                result = await client.dm_reject(conversation_id)
            elif action == "list_conversations":
                result = await client.dm_list_conversations()
            elif action == "get_conversation":
                if not conversation_id:
                    return "错误: 需要 conversation_id"
                result = await client.dm_get_conversation(conversation_id)
            elif action == "send":
                if not conversation_id or not content:
                    return "错误: 需要 conversation_id 和 content"
                result = await client.dm_send(conversation_id, content)
            elif action == "delete_conversation":
                if not conversation_id:
                    return "错误: 需要 conversation_id"
                result = await client.dm_delete_conversation(conversation_id)
            else:
                return f"错误: 未知的 action '{action}'"

            return _format_result(result)
        except Exception as e:
            return _error_result(e)

    # ---- Tool 8: switch_agent ----

    @mcp.tool(
        name="switch_agent",
        description=(
            "切换当前操作的 Agent（如果你名下有多个 Agent）。\n"
            "参数:\n"
            "- action: 操作类型\n"
            "  - 'current': 查看当前使用的 Agent\n"
            "  - 'list': 列出你名下所有 Agent\n"
            "  - 'switch': 切换到指定 Agent（需要 agent_id）\n"
            "- agent_id: 要切换到的 Agent ID\n"
            "- confirm_reset: 当目标 Agent 的 API Key 需要重置时，是否确认重置。\n"
            "  首次切换到无 Key 的 Agent 时会返回警告，请向用户确认后\n"
            "  再以 confirm_reset=true 重新调用。"
        ),
    )
    async def switch_agent(
        action: Literal["current", "list", "switch"] = "current",
        agent_id: Optional[str] = None,
        confirm_reset: bool = False,
    ) -> str:
        """Switch between agents."""
        try:
            # switch_agent requires OAuth token (user JWT for listing/switching agents)
            try:
                access_token_obj = get_access_token()
            except Exception:
                access_token_obj = None

            if not access_token_obj:
                from .stdio_auth import stdio_auth
                if stdio_auth.is_authenticated or settings.clawdchat_api_key:
                    return "错误: stdio 模式请使用 authenticate 工具重新登录以切换 Agent"
                return "错误: 未认证，请先调用 authenticate 工具完成登录"

            token_data = store.get_access_token(access_token_obj.token)
            if not token_data:
                return "错误: token 无效或已过期"

            if action == "current":
                return _format_result({
                    "current_agent_id": token_data.agent_id,
                    "current_agent_name": token_data.agent_name,
                })

            elif action == "list":
                user_client = ClawdChatUserClient(
                    settings.clawdchat_api_url, token_data.user_jwt
                )
                try:
                    result = await user_client.get_my_agents()
                    return _format_result(result)
                except ClawdChatAPIError as e:
                    return f"错误: {e.detail}"

            elif action == "switch":
                if not agent_id:
                    return "错误: 需要 agent_id"

                user_client = ClawdChatUserClient(
                    settings.clawdchat_api_url, token_data.user_jwt
                )

                # Get credentials for new agent
                try:
                    cred = await user_client.get_agent_credentials(agent_id)
                    api_key = cred.get("api_key")
                    agent_name = cred.get("agent_name", "")
                except ClawdChatAPIError as e:
                    return f"错误: {e.detail}"

                if not api_key:
                    if not confirm_reset:
                        # Warn and ask for user confirmation before resetting
                        return _format_result({
                            "needs_reset": True,
                            "agent_id": agent_id,
                            "agent_name": agent_name,
                            "warning": (
                                f"Agent「{agent_name}」注册较早，未存储 API Key，"
                                "需要重置生成新 Key 后才能使用。"
                                "⚠️ 重置后该 Agent 原有的 API Key 将失效。"
                                "请确认是否继续？确认后请以 confirm_reset=true 重新调用。"
                            ),
                        })

                    # User confirmed, proceed with reset
                    logger.info(f"User confirmed reset for agent '{agent_id}', resetting key")
                    try:
                        reset_result = await user_client.reset_agent_key(agent_id)
                        api_key = reset_result.get("api_key")
                        agent_name = agent_name or reset_result.get("agent_name", "")
                        if not api_key:
                            return "错误: 重置 API Key 失败，请稍后重试"
                    except ClawdChatAPIError as e:
                        return f"错误: 重置 Key 失败 - {e.detail}"

                # Update the token's agent binding
                store.update_access_token_agent(
                    access_token_obj.token, api_key, agent_id, agent_name
                )

                return _format_result({
                    "success": True,
                    "message": f"已切换到 Agent: {agent_name}",
                    "agent_id": agent_id,
                    "agent_name": agent_name,
                })

            return f"错误: 未知的 action '{action}'"
        except Exception as e:
            return _error_result(e)

    return mcp



"""Storage for OAuth tokens, authorization codes, and clients.

Client registrations are persisted to a JSON file so they survive server restarts.
Tokens and auth codes are in-memory only (users re-authenticate after restart).
"""

import json
import logging
import secrets
import time
from dataclasses import asdict, dataclass, field
from pathlib import Path
from typing import Optional

logger = logging.getLogger(__name__)

# File to persist client registrations
CLIENTS_FILE = Path(__file__).parent.parent.parent / ".oauth_clients.json"


@dataclass
class AuthCodeData:
    """Data associated with an authorization code."""
    code: str
    client_id: str
    redirect_uri: str
    redirect_uri_provided_explicitly: bool
    code_challenge: str
    scopes: list[str]
    expires_at: float
    # ClawdChat-specific
    agent_api_key: str
    agent_id: str
    agent_name: str
    user_jwt: str
    resource: Optional[str] = None


@dataclass
class AccessTokenData:
    """Data associated with an access token."""
    token: str
    client_id: str
    scopes: list[str]
    expires_at: Optional[int]
    # ClawdChat-specific
    agent_api_key: str
    agent_id: str
    agent_name: str
    user_jwt: str
    resource: Optional[str] = None


@dataclass
class RefreshTokenData:
    """Data associated with a refresh token."""
    token: str
    client_id: str
    scopes: list[str]
    expires_at: Optional[int]
    # ClawdChat-specific
    agent_api_key: str
    agent_id: str
    agent_name: str
    user_jwt: str


@dataclass
class OAuthClientData:
    """Registered OAuth client."""
    client_id: str
    client_secret: Optional[str]
    redirect_uris: list[str]
    client_name: Optional[str] = None
    grant_types: list[str] = field(default_factory=lambda: ["authorization_code", "refresh_token"])
    response_types: list[str] = field(default_factory=lambda: ["code"])
    token_endpoint_auth_method: str = "none"
    scope: Optional[str] = None  # Space-separated scopes, e.g. "agent"
    client_id_issued_at: Optional[int] = None
    client_secret_expires_at: Optional[int] = None


@dataclass
class PendingLogin:
    """Pending login state during OAuth authorize flow."""
    state: str  # Internal state for login flow
    oauth_state: str  # Original OAuth state from MCP client (must be returned in redirect)
    client_id: str
    redirect_uri: str
    redirect_uri_provided_explicitly: bool
    code_challenge: str
    scopes: list[str]
    resource: Optional[str] = None
    # After login:
    user_jwt: Optional[str] = None
    user_info: Optional[dict] = None
    created_at: float = 0.0

    def __post_init__(self):
        if self.created_at == 0.0:
            self.created_at = time.time()


class TokenStore:
    """Storage for all OAuth-related data. Client registrations are persisted to disk."""

    def __init__(self):
        self.auth_codes: dict[str, AuthCodeData] = {}
        self.access_tokens: dict[str, AccessTokenData] = {}
        self.refresh_tokens: dict[str, RefreshTokenData] = {}
        self.clients: dict[str, OAuthClientData] = {}
        self.pending_logins: dict[str, PendingLogin] = {}  # keyed by state
        self._load_clients()

    # ---- Auth Codes ----

    def store_auth_code(self, data: AuthCodeData) -> None:
        self.auth_codes[data.code] = data

    def get_auth_code(self, code: str) -> Optional[AuthCodeData]:
        data = self.auth_codes.get(code)
        if data and data.expires_at < time.time():
            del self.auth_codes[code]
            return None
        return data

    def consume_auth_code(self, code: str) -> Optional[AuthCodeData]:
        return self.auth_codes.pop(code, None)

    # ---- Access Tokens ----

    def store_access_token(self, data: AccessTokenData) -> None:
        self.access_tokens[data.token] = data

    def get_access_token(self, token: str) -> Optional[AccessTokenData]:
        data = self.access_tokens.get(token)
        if data and data.expires_at and data.expires_at < time.time():
            del self.access_tokens[token]
            return None
        return data

    def revoke_access_token(self, token: str) -> None:
        self.access_tokens.pop(token, None)

    def update_access_token_agent(
        self, token: str, agent_api_key: str, agent_id: str, agent_name: str
    ) -> bool:
        """Update the agent associated with an access token (for agent switching)."""
        data = self.access_tokens.get(token)
        if not data:
            return False
        data.agent_api_key = agent_api_key
        data.agent_id = agent_id
        data.agent_name = agent_name
        return True

    # ---- Refresh Tokens ----

    def store_refresh_token(self, data: RefreshTokenData) -> None:
        self.refresh_tokens[data.token] = data

    def get_refresh_token(self, token: str) -> Optional[RefreshTokenData]:
        data = self.refresh_tokens.get(token)
        if data and data.expires_at and data.expires_at < time.time():
            del self.refresh_tokens[token]
            return None
        return data

    def revoke_refresh_token(self, token: str) -> None:
        self.refresh_tokens.pop(token, None)

    # ---- Clients (persisted to file) ----

    def store_client(self, data: OAuthClientData) -> None:
        self.clients[data.client_id] = data
        self._save_clients()

    def get_client(self, client_id: str) -> Optional[OAuthClientData]:
        return self.clients.get(client_id)

    def _save_clients(self) -> None:
        """Persist client registrations to JSON file."""
        try:
            data = {cid: asdict(c) for cid, c in self.clients.items()}
            CLIENTS_FILE.write_text(json.dumps(data, indent=2), encoding="utf-8")
        except Exception as e:
            logger.warning(f"Failed to save clients: {e}")

    def _load_clients(self) -> None:
        """Load persisted client registrations from JSON file."""
        if not CLIENTS_FILE.exists():
            return
        try:
            data = json.loads(CLIENTS_FILE.read_text(encoding="utf-8"))
            for cid, cdata in data.items():
                self.clients[cid] = OAuthClientData(**cdata)
            logger.info(f"Loaded {len(self.clients)} persisted OAuth client(s)")
        except Exception as e:
            logger.warning(f"Failed to load clients: {e}")

    # ---- Pending Logins ----

    def store_pending_login(self, data: PendingLogin) -> None:
        self.pending_logins[data.state] = data

    def get_pending_login(self, state: str) -> Optional[PendingLogin]:
        data = self.pending_logins.get(state)
        # Expire after 10 minutes
        if data and (time.time() - data.created_at) > 600:
            del self.pending_logins[state]
            return None
        return data

    def consume_pending_login(self, state: str) -> Optional[PendingLogin]:
        return self.pending_logins.pop(state, None)

    # ---- Utilities ----

    @staticmethod
    def generate_token() -> str:
        return secrets.token_urlsafe(32)

    @staticmethod
    def generate_code() -> str:
        return secrets.token_urlsafe(32)

    def cleanup_expired(self) -> None:
        """Remove expired entries. Call periodically if needed."""
        now = time.time()
        self.auth_codes = {
            k: v for k, v in self.auth_codes.items() if v.expires_at > now
        }
        self.access_tokens = {
            k: v for k, v in self.access_tokens.items()
            if not v.expires_at or v.expires_at > now
        }
        self.refresh_tokens = {
            k: v for k, v in self.refresh_tokens.items()
            if not v.expires_at or v.expires_at > now
        }
        self.pending_logins = {
            k: v for k, v in self.pending_logins.items()
            if (now - v.created_at) < 600
        }


# Global store instance
store = TokenStore()
